const nsIFile                           = Components.interfaces.nsIFile;
const nsILocalFile                      = Components.interfaces.nsILocalFile;
const nsISupportsString                 = Components.interfaces.nsISupportsString;
const nsITimer                          = Components.interfaces.nsITimer;

const CACHE_FIXER_TIMER_CONTRACTID      = Components.classes["@mozilla.org/timer;1"];

const CACHE_FIXER_CONSOLE_SERVICE       = Components.classes["@mozilla.org/consoleservice;1"]
                                                    .getService(Components.interfaces.nsIConsoleService);

const CACHE_FIXER_PREFSERVICE           = Components.classes["@mozilla.org/preferences-service;1"]
                                                    .getService(Components.interfaces.nsIPrefService);

const CACHE_FIXER_PREFSERVICE_BRANCH    = CACHE_FIXER_PREFSERVICE.getBranch("browser.cache.disk.");

const CACHE_FIXER_OBSERVER_SERVICE      = Components.classes["@mozilla.org/observer-service;1"]
                                                    .getService(Components.interfaces.nsIObserverService);

const CACHE_FIXER_DIR_LOCATOR           = Components.classes["@mozilla.org/file/directory_service;1"]
                                                    .getService(Components.interfaces.nsIProperties);

function nsCacheFixer() {
    CACHE_FIXER_OBSERVER_SERVICE.addObserver(this,"profile-after-change",false);
    this._firstTime = true;
}

nsCacheFixer.prototype = {

    _dump: function(msg) {
        if (!this._debug) return;
        msg = "[Cache Fixer:] " + msg + "\n";
        CACHE_FIXER_CONSOLE_SERVICE.logStringMessage(msg);
    },
    
    _getStringBundle: function () {
        if (!this.stringBundle) {
            this.stringBundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
                                          .createInstance(Components.interfaces.nsIStringBundleService)
                                          .createBundle("chrome://cachefixer/locale/cachefixer.properties");
        }
        return this.stringBundle;
    },
    
    _getString: function (name) {
        return this._getStringBundle().GetStringFromName(name);
    },
    
    _localizingDescription: function() {
        //~ "Goofy-method", thanks Luana aka MatrixIsAllOver (extenzilla.it)
        var cfDescription = Components.classes["@mozilla.org/supports-string;1"]
                                      .createInstance(nsISupportsString);
        cfDescription.data = this._getString("initialDescription");
        
        CACHE_FIXER_PREFSERVICE.getBranch("extensions.{afcc0bd2-0b43-4a08-9981-afd8f3520e64}.")
                               .setComplexValue("description", nsISupportsString, cfDescription);
    },
    
    _setTimer: function() {
        if (this._timer) {
            this._timer.cancel();
        } else {
            this._timer = CACHE_FIXER_TIMER_CONTRACTID.createInstance(nsITimer);
        }
        
	if (this._timeOut !=0 )
	    this._timer.initWithCallback(this, this._timeOut, nsITimer.TYPE_REPEATING_SLACK);
    },
    
    _firstStart: function() {
        this._localizingDescription(); //~ localizing initial description here
	this._setTimer();
        this._firstTime = false;
    },
    
    //~ nsITimer
    notify: function() {
        CACHE_FIXER_OBSERVER_SERVICE.notifyObservers(null, "profile-before-change", false);
        CACHE_FIXER_OBSERVER_SERVICE.notifyObservers(null, "profile-after-change", false);
    },
    
    _loadSettings: function() {
        if (this._settingsLoaded) return true;
	
        try {
            this._debug = CACHE_FIXER_PREFSERVICE_BRANCH.getBoolPref("cache_fixer_debug");
            this._timeOut = CACHE_FIXER_PREFSERVICE_BRANCH.getIntPref("cache_fixer_save_timeout")*60000;
        } catch(e) {
            this._debug = false;
            this._timeOut = 0;
        }
        
        this._dump(this._getString("ReLoadSettings"));
        this._dump(this._getString("SaveTimeOut") + ": " + this._timeOut);
        
        this._cacheMapFile = null;
        
        var cacheDir, cacheMapFile;
        
        try { //~ prefs
            cacheDir = CACHE_FIXER_PREFSERVICE_BRANCH.getCharPref("parent_directory");
            this._dump(this._getString("ParentCacheDirFound") + ": " + cacheDir);
        } catch(e) {
            try { //~ FF1.1
                cacheDir = CACHE_FIXER_DIR_LOCATOR.get("ProfLD", nsIFile).path;
                this._dump(this._getString("ParentCacheDirFound") + ": " + cacheDir);
            } catch(e) {
                try { //~ FF1.0-
                    cacheDir = CACHE_FIXER_DIR_LOCATOR.get("ProfD", nsIFile).path;
                    this._dump(this._getString("ParentCacheDirFound") + ": " + cacheDir);
                } catch(e) {
                    this._dump(this._getString("ParentCacheDirNotFound"));
                    return false;
                }
            }
        }
        
        if (cacheDir) {
            cacheMapFile = Components.classes["@mozilla.org/file/local;1"].createInstance(nsILocalFile);
            cacheMapFile.initWithPath(cacheDir);
            cacheMapFile.append("Cache");
            cacheMapFile.append("_CACHE_MAP_");
        }
        if (!cacheMapFile || !cacheMapFile.exists()) {
            this._dump(this._getString("CacheMapFileNotFound"));
            return false;
        }
	
	this._cacheMapFile = cacheMapFile;
	
        this._settingsLoaded = true;
        
	return true;
    },
    
    //~ nsIObserver interface
    observe: function(aSubject, aTopic, aData) {
        
        if (aTopic == "profile-after-change") {
        
	    if (aData) this._settingsLoaded = false;
            
            if (!this._loadSettings()) return;
	    if (this._firstTime) this._firstStart();
	    
	    var strip = "";
	    var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	    var fileHandler = ios.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
	    var URL = fileHandler.getURLSpecFromFile(this._cacheMapFile);
	    var uri = ios.newURI(URL,"utf8",null);
	    var channel = ios.newChannelFromURI(uri);
	    var binInput = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
	    binInput.setInputStream(channel.open());
	    var strip = binInput.readBytes(15);
            binInput.close();
            
	    if (strip.length == 15) {
	        var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		stream.init(this._cacheMapFile, 0x02 , 0644, 0);
		stream.write(strip , strip.length);
		stream.write("\x00", 1);
		stream.close();
		this._dump(this._getString("CachePatched"));
	    }
	}
	
	else if (aTopic == "nsPref:changed") {
	
	    try {this._timeOut=CACHE_FIXER_PREFSERVICE_BRANCH.getIntPref("cache_fixer_save_timeout")*60000;}
	    catch(e) {this._timeOut=0;}
	    
            this._settingsLoaded = false;
	    this._setTimer();
        
	}
        
        else if (aTopic == "app-startup") {
        
            var prefInternal = Components.classes["@mozilla.org/preferences-service;1"]
                                         .getService(Components.interfaces.nsIPrefBranchInternal);
            prefInternal.addObserver("browser.cache.disk.", this, true);
        
        }
    },
    
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(Components.interfaces.nsISupportsWeakReference) &&
            !iid.equals(Components.interfaces.nsIObserver))
        {
            dump("nsCacheFixer: QI unknown interface: " + iid + "\n");
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    },

    //~ This Component's module implementation.
    module: {
        
        // registerSelf: Register this component.
        registerSelf: function (compMgr, fileSpec, location, type) {
            var compReg = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
            compReg.registerFactoryLocation( this.cid,
                                             "nsCacheFixer JS component",
                                             this.contractId,
                                             fileSpec, location, type );
            
            var catman = Components.classes["@mozilla.org/categorymanager;1"]
                                   .getService(Components.interfaces.nsICategoryManager);
            catman.addCategoryEntry("app-startup", "nsCacheFixer", this.contractId, true, true);
        },
        
        unregisterSelf: function(compMgr, fileSpec, location) {
            compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
            compMgr.unregisterFactoryLocation( this.cid, fileSpec );
            var catman = Components.classes["@mozilla.org/categorymanager;1"]
                                   .getService(Components.interfaces.nsICategoryManager);
            catman.deleteCategoryEntry("app-startup", this.contractId, true);
        },
        
        // getClassObject: Return this component's factory object.
        getClassObject: function (compMgr, cid, iid) {
            if (!cid.equals(this.cid))
                throw Components.results.NS_ERROR_NO_INTERFACE;
            
            if (!iid.equals(Components.interfaces.nsIFactory))
                throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
            
            return this.factory;
        },
        
        /* CID for this class */
        //~ extensions.roachfiend.com/cgi-bin/guid.pl
        cid: Components.ID("{ddbfe606-9b5a-4780-af06-38fc3705a3b8}"),
        
        /* Contract ID for this class */
        contractId: "@mozilla.org/cachefixer;1",
        
        /* factory object */
        factory: {
            // createInstance: Return a new nsCacheFixer object.
            createInstance: function (outer, iid) {
                if (outer != null)
                    throw Components.results.NS_ERROR_NO_AGGREGATION;
                
                return (new nsCacheFixer()).QueryInterface(iid);
            }
        },
        
        // canUnload: n/a (returns true)
        canUnload: function(compMgr) {
            return true;
        }
    }
}

// NSGetModule: Return the nsIModule object.
function NSGetModule(compMgr, fileSpec) {
    return nsCacheFixer.prototype.module;
}